#! /bin/sh

if [ -f /etc/syslog.conf ]        # Log to syslog if available...
then
    exec >/tmp/$$.tmp 2>&1
else
    exec >/dev/console 2>&1
fi

# first parameter is a shell script containing all parameters needed
# second parameter is action
# 	new   a new ip is assigned
#	up    the same as stored in .cache file is assigned
# 	down  interface goes down
# third parameter is debug (-d) if dhcpcd was started with -d
config=$1
action=$2
debug=$3

if [ "$debug" = "-d" ]
then
    set -x
fi    

set `cat /var/run/xdsl-tunnel`
inode=$1
inode_if=$2
rm -f /var/run/${inode_if}_down


if [ ! -f "$config" ]
then
    exit 0
fi
. $config

if [ "$INTERFACE" -a "$action" ]
then
    if [ "$INTERFACE" = "$inode_if" ]
    then
	case "$action" in
	new)
	    ifconfig $inode_if $IPADDR netmask $NETMASK broadcast $BROADCAST
	    route add -host $1 gw $GATEWAY
	    ifconfig $inode_if
	    route -n
	    ;;
	up)
	    ifconfig $inode_if
	    route -n
	    ifconfig $inode_if $IPADDR netmask $NETMASK broadcast $BROADCAST
	    route add -host $1 gw $GATEWAY
	    ifconfig $inode_if
	    route -n
	    ;;
	down)
	    ifconfig $inode_if down
	    ;;
	*)
	    echo "unknown request $action for interface $INTERFACE"
	esac
    else
	echo "requested action $action for unknown interface $INTERFACE"
    fi
fi


if [ -f  /tmp/$$.tmp ]
then
    logger -t dhcpcd < /tmp/$$.tmp
    rm /tmp/$$.tmp
fi
