#! /bin/sh

usage ()
{
    cat <<EOF 
$script [setup|start|stop|dial|hangup|dialmode|huptimeout] \\
	    <circuit name> <params>
    setup <circuit name> <interface> [async|sync|in_kernel] [auto|manual] \\
	  <user> <passwd> <idle timeout> <usepeerdns: [yes|no]> \\
	  <active filter: [yes|no]> <debug: [yes|no]>
    start <circuit name>
    dial <circuit name>
    hangup <circuit name>
    dialmode <circuit name> [auto|manual]
    huptimeout <number>
    stop <circuit name>
EOF
    exit 1
}

unimplemented ()
{
    echo "$0: function $1 currently unimplemented"
    exit 1
}

imond_name ()
{
    if [ -f /var/run/imond.pid ]
    then
	imond-stat "pppoe: name:$1"
    fi
}
setup ()
{
    # $script setup $circuit $interface async|sync|in_kernel $dialmode $user $passwd  $hup_timeout <yes|no> <yes|no> <yes|no>
    interface=$1
    mode=$2
    dialmode=$3
    user=$4
    passwd=$5
    hup_timeout=$6

    if [ "$7" = 'yes' ]
    then
	usepeerdns='usepeerdns'
	echo yes > /var/run/$circuit.usepeerdns
    fi
    if [ "$8" = 'yes' ]
    then
	active_filter='precompiled-active-filter /etc/ppp/filter'
    fi
    if [ "$9" = 'yes' ]
    then
	debug='debug'
    fi

    clampmss="-m 1412"                              # clamp the MSS
    lcp_interval=30                                 # LCP interval
    lcp_failure=5                                   # LCP failure count
    pppoe_timeout=120                               # should be 4 * lcp_interval
    local=10.112.112.112
    remote=10.112.112.113
    netmask=255.255.255.0

    # load modules, build pppoe command line and pppd options
    $EXEC do_insmod_once slhc
    $EXEC do_insmod_once ppp_generic

    tty_opt="exec /usr/sbin/pppoe -p /var/run/pppoe.pid -I $interface -T $pppoe_timeout $clampmss"
    pppd_sync=''
    case $mode in
	async)
	    $EXEC do_insmod_once ppp_async
	    tty_opt="pty \"$tty_opt\""
	    ;;
	sync)
	    $EXEC do_insmod_once ppp_synctty
	    $EXEC do_insmod_once n_hdlc
	    tty_opt="pty \"$tty_opt -s\""
	    pppd_sync='sync'
	    ;;
	in_kernel)
	    $EXEC do_insmod_once pppox
	    $EXEC do_insmod_once pppoe
	    tty_opt='plugin rp-pppoe.so'
	    ;;
	*)
	    log_error "unknown mode"
	    ;;
    esac

    # build /etc/ppp/peers/$circuit
    cat <<-EOF > $PREFIX/etc/ppp/peers/$circuit.manual
	$usepeerdns
	$debug
	$local:$remote
	user "$user"
	lcp-echo-interval $lcp_interval
	lcp-echo-failure $lcp_failure
	$tty_opt
	linkname $circuit
	defaultroute
	noipdefault
	noauth
	hide-password
	nodetach
	local
	mtu 1492
	mru 1492
	noaccomp
	noccp
	nobsdcomp
	nodeflate
	nopcomp
	novj
	novjccomp
	holdoff 3
	maxfail 0
	ipcp-accept-remote
	ipcp-accept-local
	connect true
	ktune
EOF
    cp $PREFIX/etc/ppp/peers/$circuit.manual $PREFIX/etc/ppp/peers/$circuit.auto
    if [ 0$hup_timeout -eq 0 ]
    then
	echo "persist" >> $PREFIX/etc/ppp/peers/$circuit.auto 
    else
	cat <<EOF >> $PREFIX/etc/ppp/peers/$circuit.manual
idle $hup_timeout
$active_filter
EOF
	cat <<EOF >> $PREFIX/etc/ppp/peers/$circuit.auto
idle $hup_timeout
$active_filter
demand
EOF
    fi
    # write /etc/ppp/pap-secrets-template
    echo "\"$user\"  *  \"$passwd\"  *" > $PREFIX/etc/ppp/peers/$circuit.pap-secrets.tmpl
    # remember some circuit information
    echo "$interface $mode" > $PREFIX/etc/ppp/peers/$circuit.info
    # set dialmode
    echo $dialmode > $PREFIX/var/run/$circuit.dialmode
}

really_start_pppd ()
{
    cp $PREFIX/etc/ppp/pap-secrets $PREFIX/etc/ppp/pap-secrets.orig
    cat $PREFIX/etc/ppp/peers/$circuit.pap-secrets.tmpl >> $PREFIX/etc/ppp/pap-secrets
    set `cat $PREFIX/etc/ppp/peers/$circuit.info`
    interface=$1
    mode=$2
    loop='yes'

    $EXEC ifconfig $interface up mtu 1500                 # set mtu to 1500
    > $PREFIX/var/run/$circuit.started
    echo "$circuit pppoe" > $PREFIX/var/run/circuit-name.$interface
    
    while true
    do
	dialmode=`cat $PREFIX/var/run/$circuit.dialmode`
	if [ "$mode" = 'in_kernel' ]
	then
	    $EXEC pppd call $circuit.$dialmode $interface
	else
	    $EXEC pppd call $circuit.$dialmode
	fi
	if [ ! -f $PREFIX/var/run/$circuit.restart ]
	then
	    break
	fi
    done
    rm -f $PREFIX/var/run/$circuit.started
    cat $PREFIX/etc/ppp/pap-secrets.orig > $PREFIX/etc/ppp/pap-secrets
}

start_pppd ()
{
    cp $PREFIX/var/run/$circuit.usepeerdns $PREFIX/var/run/pppoe.usepeerdns
    cp $PREFIX/var/run/$circuit.dialmode $PREFIX/var/run/pppoe.dialmode
    really_start_pppd &
    while [ ! -e $PREFIX/var/run/ppp-$circuit.pid ]
    do
	sleep 1
    done

}

start ()
{
    dialmode=`cat $PREFIX/var/run/$circuit.dialmode`
    # imond_name $circuit

    if [ "$dialmode" = 'auto' -a ! -f $PREFIX/var/run/$circuit.started ]
    then
	> $PREFIX/var/run/$circuit.restart
	start_pppd
    fi
}

stop ()
{
    if [ -f $PREFIX/var/run/$circuit.started ]
    then
	rm -f $PREFIX/var/run/$circuit.restart
	set `cat $PREFIX/var/run/ppp-$circuit.pid`
	$EXEC kill $1
	while [ -e $PREFIX/var/run/ppp-$circuit.pid ]
	do
	    sleep 1
	done
	set `cat $PREFIX/etc/ppp/peers/$circuit.info`
	$EXEC ifconfig $1 down
	rm -f $PREFIX/var/run/$circuit.started $PREFIX/var/run/circuit-name.$interface
    fi
    # imond_name none
}

dial ()
{
    dialmode=`cat $PREFIX/var/run/$circuit.dialmode`
    if [ -f $PREFIX/var/run/$circuit.started ]
    then
	if [ "$dialmode" = 'auto' ]
	then
	    ping -c 1 1.2.3.4
	fi
    else
	if [ "$dialmode" = 'manual' ]
	then
	    start_pppd
	elif [ "$dialmode" = 'auto' ]
	then
	    echo "$script: circuit $circuit not started yet"
	    exit 1
	fi
    fi
}

hangup ()
{
    set `cat $PREFIX/var/run/ppp-$circuit.pid`
    $EXEC kill -HUP $1
}

dialmode ()
{
    dialmode=`cat $PREFIX/var/run/$circuit.dialmode`
    case $dialmode in
	$1) ;;
	*)  echo $1 > $PREFIX/var/run/$circuit.dialmode
	    case $1 in 
		manual) 
		    rm -f $PREFIX/var/run/$circuit.restart
		    ;;
		auto)
		    start
		    ;;
	    esac
    esac
}

huptimeout ()
{
    sed -e "s/^idle[[:space:]].*/idle $1/g" $PREFIX/etc/ppp/peers/$circuit > $PREFIX/tmp/$circuit.$$
    mv $PREFIX/tmp/$circuit.$$ $PREFIX/etc/ppp/peers/$circuit
}

vrfy_params ()
{
    while [ "$1" ]
    do
	skip='yes'
	case "$1" in
	    exists)
		if [ ! -f $PREFIX/etc/ppp/peers/$circuit.info ]
		then
		    echo "$script - $op: circuit $circuit doesn't exist"
		fi
		;;
	    notexists)
		if [ -f $PREFIX/etc/ppp/peers/$circuit.info ]
		then
		    echo "$script - $op: circuit $circuit already exists"
		fi
		;;
	    started)
		if [ ! -f $PREFIX/var/run/$circuit.started ]
		then
		    echo "$script - $op: circuit $circuit already not started yet"
		fi
		;;
	    notstarted)
		if [ -f $PREFIX/var/run/$circuit.started ]
		then
		    echo "$script - $op: circuit $circuit already started"
		fi
		;;
	    params)
		skip=''
		num=$2
		shift 2
		if [ $# -ne $num ]
		then
		    echo "$script: invalid parameter(s) '$*' for $op"
		    usage
		fi
		break 
		;;
	    *)
		echo "unknown check $*"
		exit 1
		;;
	esac
	if [ "$skip" ]
	then
	    shift 2
	fi
    done
    if [ ! -f $PREFIX/etc/ppp/peers/$circuit.info ]
    then
	echo "$script: circuit $circuit doesn't exist"
	usage
    fi
}

run_test ()
{
    cat <<EOF
#
# $*
#
EOF
    $*
}
test_script ()
{
    PREFIX='./test'
    if [ -d ../../../../../../base ]
    then
	BOOT_PREFIX=../../../../../../base/opt
    else
	BOOT_PREFIX=../../../../../opt
    fi
    EXEC=echo
    export PREFIX EXEC BOOT_PREFIX

    dialmode=$1

    run_test $0 -x setup foo eth1 in_kernel manual user passwd idle_time yes yes yes
    run_test $0 -x start $circuit
    run_test $0 -x dial $circuit
    run_test $0 -x hangup $circuit    
    run_test $0 -x stop $circuit
}

. $BOOT_PREFIX/etc/boot.d/base-helper

script=`basename $0`
if [ "$1" = '-x' ]
then
    shift
    set -x
fi
op=$1
circuit=$2
shift 2

case $op in
    setup)
	setup $@
	;;
    start)
	vrfy_params exists $circuit params 0 $*
	start $*
	;;
    stop)
	vrfy_params exists $circuit params 0 $*
	stop $*
	;;
    dial)
	vrfy_params exists $circuit params 0 $*
	dial $*
	;;
    hangup)
	vrfy_params exists $circuit params 0 $*
	hangup $*
	;;
    dialmode)
	vrfy_params exists $circuit params 1 $*
	dialmode $*
	;;
    huptimeout)
	vrfy_params exists $circuit params 1 $*
	huptimeout $*
	;;
    test)
	test_script
	;;
    *)
	usage
 	;;
esac
